@@ -2,7 +2,7 @@
  *
  * $Id: index.cpp,v 1.30 1999/03/06 22:15:39 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -51,12 +51,13 @@ static void addToMap(const char *name,Definition *d)
     }
     //printf("******* adding symbol `%s'\n",symbolName.data());
     dl->append(d);
+    d->setSymbolName(symbolName);
   }
 }
 
 static void removeFromMap(Definition *d)
 {
-  QCString symbolName = d->name();
+  QCString symbolName = d->symbolName();
   int index=symbolName.findRev("::");
   if (index!=-1) symbolName=symbolName.mid(index+2);
   if (!symbolName.isEmpty()) 
@@ -73,7 +74,8 @@ static void removeFromMap(Definition *d)
 }
 
 Definition::Definition(const char *df,int dl,
-                       const char *name,const char *b,const char *d)
+                       const char *name,const char *b,
+                       const char *d,bool isSymbol)
 {
   //QCString ns;
   m_defFileName = df;
@@ -107,12 +109,13 @@ Definition::Definition(const char *df,int dl,
   m_briefFile=(QCString)"<"+name+">";
   m_docLine=1;
   m_docFile=(QCString)"<"+name+">";
-  addToMap(name,this);
+  m_isSymbol = isSymbol;
+  if (m_isSymbol) addToMap(name,this);
 }
 
 Definition::~Definition()
 {
-  removeFromMap(this);
+  if (m_isSymbol) removeFromMap(this);
   delete m_sectionDict;
   delete m_sourceRefByDict;
   delete m_sourceRefsDict;
@@ -206,6 +209,8 @@ void Definition::setDocumentation(const char *d,const char *docFile,int docLine,
   m_docLine = docLine;
 }
 
+#define uni_isupper(c) (QChar(c).category()==QChar::Letter_Uppercase)
+
 void Definition::setBriefDescription(const char *b,const char *briefFile,int briefLine) 
 { 
   if (b==0) return;
@@ -218,7 +223,7 @@ void Definition::setBriefDescription(const char *b,const char *briefFile,int bri
     {
       case '.': case '!': case '?': break;
       default: 
-        if (isupper(m_brief.at(0))) m_brief+='.'; 
+        if (uni_isupper(m_brief.at(0))) m_brief+='.'; 
         break;
     }
   }
@@ -510,8 +515,8 @@ void Definition::writeSourceRefList(OutputList &ol,const char *scopeName,
         if (md->isFunction() || md->isSlot() || 
             md->isPrototype() || md->isSignal()
            ) name+="()";
-        Definition *d = md->getOuterScope();
-        if (d==Doxygen::globalScope) d=md->getBodyDef();
+        //Definition *d = md->getOutputFileBase();
+        //if (d==Doxygen::globalScope) d=md->getBodyDef();
         if (md->getStartBodyLine()!=-1 && md->getBodyDef()) 
         {
           //printf("md->getBodyDef()=%p global=%p\n",md->getBodyDef(),Doxygen::globalScope); 
@@ -530,12 +535,13 @@ void Definition::writeSourceRefList(OutputList &ol,const char *scopeName,
           ol.docify(name);
           ol.popGeneratorState();
         }
-        else if (md->isLinkable() && d && d->isLinkable())
+        else if (md->isLinkable() /*&& d && d->isLinkable()*/)
         {
           // for HTML write a real link
           ol.pushGeneratorState();
           ol.disableAllBut(OutputGenerator::Html);
-          ol.writeObjectLink(d->getReference(),d->getOutputFileBase(),
+          ol.writeObjectLink(md->getReference(),
+                             md->getOutputFileBase(),
                              md->anchor(),name);
           ol.popGeneratorState();
 
