@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -561,8 +561,8 @@ class DocImage : public CompAccept<DocImage>, public DocNode
 {
   public:
     enum Type { Html, Latex, Rtf };
-    DocImage(DocNode *parent,const QString &name,Type t) : 
-      m_parent(parent), m_name(name), m_type(t) {}
+    DocImage(DocNode *parent,const HtmlAttribList &attribs,const QString &name,Type t) : 
+      m_parent(parent), m_attribs(attribs), m_name(name), m_type(t) {}
     Kind kind() const          { return Kind_Image; }
     Type type() const          { return m_type; }
     QString name() const       { return m_name; }
@@ -570,11 +570,13 @@ class DocImage : public CompAccept<DocImage>, public DocNode
     bool hasCaption() const    { return !m_children.isEmpty(); }
     QString width() const      { return m_width; }
     QString height() const     { return m_height; }
+    const HtmlAttribList &attribs() const { return m_attribs; }
     void accept(DocVisitor *v) { CompAccept<DocImage>::accept(this,v); }
     void parse();
 
   private:
     DocNode *m_parent;
+    HtmlAttribList m_attribs;
     QString  m_name;
     Type     m_type;
     QString  m_width;
@@ -691,16 +693,18 @@ class DocLanguage : public CompAccept<DocLanguage>, public DocNode
 class DocHRef : public CompAccept<DocHRef>, public DocNode
 {
   public:
-    DocHRef(DocNode *parent,const QString &url) : 
-      m_parent(parent), m_url(url) {}
+    DocHRef(DocNode *parent,const HtmlAttribList &attribs,const QString &url) : 
+      m_parent(parent), m_attribs(attribs), m_url(url) {}
     int parse();
     QString url() const        { return m_url; }
     Kind kind() const          { return Kind_HRef; }
     DocNode *parent() const    { return m_parent; }
     void accept(DocVisitor *v) { CompAccept<DocHRef>::accept(this,v); }
+    const HtmlAttribList &attribs() const { return m_attribs; }
 
   private:
     DocNode * m_parent;
+    HtmlAttribList m_attribs;
     QString   m_url;
 };
 
