@@ -3,7 +3,7 @@
  * $Id: dot.cpp,v 1.20 2001/03/19 19:27:40 root Exp $
  *
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -31,6 +31,7 @@
 #include <qdir.h>
 #include <qfile.h>
 #include <qtextstream.h>
+#include <md5.h>
 
 //--------------------------------------------------------------------
 
@@ -98,10 +99,10 @@ static bool convertMapFile(QTextStream &t,const char *mapName,
   char buf[maxLineLen];
   char url[maxLineLen];
   char ref[maxLineLen];
-  bool isRef = FALSE;
   int x1,y1,x2,y2;
   while (!f.atEnd())
   {
+    bool isRef = FALSE;
     int numBytes = f.readLine(buf,maxLineLen);
     buf[numBytes-1]='\0';
     //printf("ReadLine `%s'\n",buf);
@@ -294,6 +295,36 @@ static void checkDotResult(const QCString &imgName)
   }
 }
 
+/*! Checks if a file "baseName".md5 exists. If so the contents
+ *  are compared with \a md5. If equal FALSE is returned. If the .md5
+ *  file does not exist or its contents are not equal to \a md5, 
+ *  a new .md5 is generated with the \a md5 string as contents.
+ */
+static bool checkAndUpdateMd5Signature(const QCString &baseName,const QCString &md5)
+{
+  QFile f(baseName+".md5");
+  if (f.open(IO_ReadOnly))
+  {
+    // read checksum
+    QCString md5stored(33);
+    int bytesRead=f.readBlock(md5stored.data(),32);
+    // compare checksum
+    if (bytesRead==32 && md5==md5stored)
+    {
+      // bail out if equal
+      return FALSE;
+    }
+  }
+  f.close();
+  // create checksum file
+  if (f.open(IO_WriteOnly))
+  {
+    f.writeBlock(md5.data(),32); 
+    f.close();
+  }
+  return TRUE;
+}
+
 //--------------------------------------------------------------------
 
 class DotNodeList : public QList<DotNode>
@@ -573,7 +604,7 @@ void DotNode::write(QTextStream &t,
                     bool backArrows
                    )
 {
-  //printf("DotNode::write(%d) name=%s\n",distance,m_label.data());
+  //printf("DotNode::write(%d) name=%s this=%p written=%d\n",distance,m_label.data(),this,m_written);
   if (m_written) return; // node already written to the output
   if (m_distance>distance) return;
   QList<DotNode> *nl = toChildren ? m_children : m_parents; 
@@ -600,6 +631,7 @@ void DotNode::write(QTextStream &t,
       {
         if (cn->m_distance<=distance) 
         {
+          //printf("write arrow %s%s%s\n",label().data(),backArrows?"<-":"->",cn->label().data());
           writeArrow(t,gt,format,cn,dnli2.current(),topDown,backArrows);
         }
         cn->write(t,gt,format,topDown,toChildren,distance,backArrows);
@@ -613,19 +645,21 @@ void DotNode::write(QTextStream &t,
       {
         if (pn->m_distance<=distance) 
         {
+          //printf("write arrow %s%s%s\n",label().data(),backArrows?"<-":"->",pn->label().data());
           writeArrow(t,
-                     gt,
-                     format,
-                     pn,
-                     pn->m_edgeInfo->at(pn->m_children->findRef(this)),
-                     FALSE,
-                     backArrows
-                    );
+              gt,
+              format,
+              pn,
+              pn->m_edgeInfo->at(pn->m_children->findRef(this)),
+              FALSE,
+              backArrows
+              );
         }
         pn->write(t,gt,format,TRUE,FALSE,distance,backArrows);
       }
     }
   }
+  //printf("end DotNode::write(%d) name=%s\n",distance,m_label.data());
 }
 
 void DotNode::writeXML(QTextStream &t,bool isClassGraph)
@@ -879,6 +913,7 @@ void DotGfxHierarchyTable::writeGraph(QTextStream &out,const char *path)
   QDir::setCurrent(d.absPath());
   QDir thisDir;
 
+  // put each connected subgraph of the hierarchy in a row of the HTML output
   out << "<table border=0 cellspacing=10 cellpadding=0>" << endl;
 
   QListIterator<DotNode> dnli(*m_rootSubgraphs);
@@ -890,52 +925,88 @@ void DotGfxHierarchyTable::writeGraph(QTextStream &out,const char *path)
     QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
     baseName.sprintf("inherit_graph_%d",count++);
     baseName = convertNameToFile(baseName);
-    QCString dotName=baseName+".dot";
     QCString imgName=baseName+"."+ imgExt;
     QCString mapName=baseName+".map";
-
-    QFile f(dotName);
-    if (!f.open(IO_WriteOnly)) return;
-    QTextStream t(&f);
-    writeGraphHeader(t);
-    t << "  rankdir=LR;" << endl;
     QListIterator<DotNode> dnli2(*m_rootNodes);
     DotNode *node;
-    for (;(node=dnli2.current());++dnli2)
+
+    // compute md5 checksum of the graph were are about to generate
+    QString buf;
+    QTextStream md5stream(&buf,IO_WriteOnly);
+    for (dnli2.toFirst();(node=dnli2.current());++dnli2)
     {
       if (node->m_subgraphId==n->m_subgraphId) 
-        node->write(t,DotNode::Hierarchy,BITMAP,FALSE,TRUE,1000,TRUE);
+      {
+        node->clearWriteFlag();
+      }
     }
-    writeGraphFooter(t);
-    f.close();
+    for (dnli2.toFirst();(node=dnli2.current());++dnli2)
+    {
+      if (node->m_subgraphId==n->m_subgraphId) 
+      {
+        node->write(md5stream,DotNode::Hierarchy,BITMAP,FALSE,TRUE,1000,TRUE);
+      }
+    }
+    uchar md5_sig[16];
+    QCString sigStr(33);
+    MD5Buffer((const unsigned char *)buf.ascii(),buf.length(),md5_sig);
+    MD5SigToString(md5_sig,sigStr.data(),33);
+    if (checkAndUpdateMd5Signature(baseName,sigStr) || 
+        !QFileInfo(mapName).exists())
+    { 
+      // image was new or has changed
+      QCString dotName=baseName+".dot";
+      QFile f(dotName);
+      if (!f.open(IO_WriteOnly)) return;
+      QTextStream t(&f);
+      writeGraphHeader(t);
+      t << "  rankdir=LR;" << endl;
+      for (dnli2.toFirst();(node=dnli2.current());++dnli2)
+      {
+        if (node->m_subgraphId==n->m_subgraphId) 
+        {
+          node->clearWriteFlag();
+        }
+      }
+      for (dnli2.toFirst();(node=dnli2.current());++dnli2)
+      {
+        if (node->m_subgraphId==n->m_subgraphId) 
+        {
+          node->write(t,DotNode::Hierarchy,BITMAP,FALSE,TRUE,1000,TRUE);
+        }
+      }
+      writeGraphFooter(t);
+      f.close();
 
-    QCString dotArgs(maxCmdLine);
-    dotArgs.sprintf("-T%s \"%s\" -o \"%s\"",
-           imgExt.data(), dotName.data(),imgName.data());
-    //printf("Running: dot -T%s %s -o %s\n",imgExt.data(),dotName.data(),imgName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
-    {
-      err("Problems running dot. Check your installation!\n");
-      out << "</table>" << endl;
-      return;
-    }
-    checkDotResult(imgName);
-    dotArgs.sprintf("-Timap \"%s\" -o \"%s\"",dotName.data(),mapName.data());
-    //printf("Running: dot -Timap %s -o %s\n",dotName.data(),mapName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
-    {
-      err("Problems running dot. Check your installation!\n");
-      out << "</table>" << endl;
-      return;
+      QCString dotArgs(maxCmdLine);
+      dotArgs.sprintf("-T%s \"%s\" -o \"%s\"",
+          imgExt.data(), dotName.data(),imgName.data());
+      //printf("Running: dot -T%s %s -o %s\n",imgExt.data(),dotName.data(),imgName.data());
+      if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+      {
+        err("Problems running dot. Check your installation!\n");
+        out << "</table>" << endl;
+        return;
+      }
+      checkDotResult(imgName);
+      dotArgs.sprintf("-Timap \"%s\" -o \"%s\"",dotName.data(),mapName.data());
+      //printf("Running: dot -Timap %s -o %s\n",dotName.data(),mapName.data());
+      if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+      {
+        err("Problems running dot. Check your installation!\n");
+        out << "</table>" << endl;
+        return;
+      }
+      if (Config_getBool("DOT_CLEANUP")) thisDir.remove(dotName);
     }
+    // write image and map in a table row
     QCString mapLabel = convertNameToFile(n->m_label);
     out << "<tr><td><img src=\"" << imgName << "\" border=\"0\" alt=\"\" usemap=\"#" 
       << mapLabel << "_map\">" << endl;
     out << "<map name=\"" << mapLabel << "_map\">" << endl;
     convertMapFile(out,mapName);
     out << "</map></td></tr>" << endl;
-    if (Config_getBool("DOT_CLEANUP")) thisDir.remove(dotName);
-    thisDir.remove(mapName);
+    //thisDir.remove(mapName);
   }
   out << "</table>" << endl;
 
@@ -950,11 +1021,11 @@ void DotGfxHierarchyTable::addHierarchy(DotNode *n,ClassDef *cd,bool hideSuper)
   for ( ; (bcd=bcli.current()) ; ++bcli )
   {
     ClassDef *bClass=bcd->classDef; 
-    //printf("Trying super class=`%s'\n",bClass->name().data());
+    //printf("  Trying super class=`%s' usedNodes=%d\n",bClass->name().data(),m_usedNodes->count());
     if (bClass->isVisibleInHierarchy() && hasVisibleRoot(bClass->baseClasses()))
     {
       DotNode *bn;
-      //printf("Node `%s' Found visible class=`%s'\n",n->m_label.data(),
+      //printf("  Node `%s' Found visible class=`%s'\n",n->m_label.data(),
       //                                              bClass->name().data());
       if ((bn=m_usedNodes->find(bClass->name()))) // node already present 
       {
@@ -962,7 +1033,7 @@ void DotGfxHierarchyTable::addHierarchy(DotNode *n,ClassDef *cd,bool hideSuper)
         {
           n->addChild(bn,bcd->prot);
           bn->addParent(n);
-          //printf("Adding node %s to existing base node %s (c=%d,p=%d)\n",
+          //printf("  Adding node %s to existing base node %s (c=%d,p=%d)\n",
           //       n->m_label.data(),
           //       bn->m_label.data(),
           //       bn->m_children ? bn->m_children->count() : 0,
@@ -971,7 +1042,7 @@ void DotGfxHierarchyTable::addHierarchy(DotNode *n,ClassDef *cd,bool hideSuper)
         }
         //else
         //{
-        //  printf("Class already has an arrow!\n");
+        //  printf("  Class already has an arrow!\n");
         //}
       }
       else 
@@ -983,14 +1054,15 @@ void DotGfxHierarchyTable::addHierarchy(DotNode *n,ClassDef *cd,bool hideSuper)
              bClass->displayName(),
              tmp_url.data()
            );
-        //printf("Adding node %s to new base node %s (c=%d,p=%d)\n",
+        n->addChild(bn,bcd->prot);
+        bn->addParent(n);
+        //printf("  Adding node %s to new base node %s (c=%d,p=%d)\n",
         //   n->m_label.data(),
         //   bn->m_label.data(),
         //   bn->m_children ? bn->m_children->count() : 0,
         //   bn->m_parents  ? bn->m_parents->count()  : 0
         //  );
-        n->addChild(bn,bcd->prot);
-        bn->addParent(n);
+        //printf("  inserting %s (%p)\n",bClass->name().data(),bn);
         m_usedNodes->insert(bClass->name(),bn); // add node to the used list
       }
       if (!bClass->visited && !hideSuper && bClass->subClasses()->count()>0)
@@ -1001,6 +1073,7 @@ void DotGfxHierarchyTable::addHierarchy(DotNode *n,ClassDef *cd,bool hideSuper)
       }
     }
   }
+  //printf("end addHierarchy\n");
 }
 
 void DotGfxHierarchyTable::addClassList(ClassSDict *cl)
@@ -1382,15 +1455,64 @@ void writeDotGraph(DotNode *root,
   }
 }
 
-static void findMaximalDotGraph(DotNode *root,
+/*! Computes a 16 byte md5 checksum for a given dot graph.
+ *  The md5 checksum is returned as a 32 character ASCII string.
+ */
+QCString computeMd5Signature(DotNode *root,
+                   DotNode::GraphType gt,
+                   GraphOutputFormat format,
+                   bool lrRank,
+                   bool renderParents,
+                   int distance,
+                   bool backArrows
+                  )
+{
+  //printf("computeMd5Signature\n");
+  QString buf;
+  QTextStream md5stream(&buf,IO_WriteOnly);
+  if (lrRank)
+  {
+    md5stream << "rankdir=LR;" << endl;
+  }
+  root->clearWriteFlag();
+  root->write(md5stream,gt,format,gt!=DotNode::CallGraph,TRUE,distance,backArrows);
+  if (renderParents && root->m_parents) 
+  {
+    QListIterator<DotNode>  dnli(*root->m_parents);
+    DotNode *pn;
+    for (dnli.toFirst();(pn=dnli.current());++dnli)
+    {
+      if (pn->m_distance<=distance) 
+      {
+        root->writeArrow(md5stream,
+            gt,
+            format,
+            pn,
+            pn->m_edgeInfo->at(pn->m_children->findRef(root)),
+            FALSE,
+            backArrows
+            );
+      }
+      pn->write(md5stream,gt,format,TRUE,FALSE,distance,backArrows);
+    }
+  }
+  uchar md5_sig[16];
+  QCString sigStr(33);
+  MD5Buffer((const unsigned char *)buf.ascii(),buf.length(),md5_sig);
+  MD5SigToString(md5_sig,sigStr.data(),33);
+  //printf("md5: %s | file: %s\n",sigStr,baseName.data());
+  return sigStr;
+}
+
+static bool findMaximalDotGraph(DotNode *root,
                                 int maxDist,
                                 const QCString &baseName,
                                 QDir &thisDir,
                                 DotNode::GraphType gt,
                                 GraphOutputFormat format,
-                                bool lrRank=FALSE,
-                                bool renderParents=FALSE,
-                                bool backArrows=TRUE
+                                bool lrRank /*=FALSE*/,
+                                bool renderParents /*=FALSE*/,
+                                bool backArrows /*=TRUE*/
                                )
 {
   int minDistance=1; // min distance that shows only direct children.
@@ -1417,7 +1539,7 @@ static void findMaximalDotGraph(DotNode *root,
     if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
     {
       err("Problems running dot. Check your installation!\n");
-      return;
+      return FALSE;
     }
 
     // extract bounding box from the result
@@ -1461,6 +1583,7 @@ static void findMaximalDotGraph(DotNode *root,
                 lastFit,
                 backArrows
                );
+  return TRUE;
 }
 
 QCString DotClassGraph::diskName() const
@@ -1520,105 +1643,130 @@ QCString DotClassGraph::writeGraph(QTextStream &out,
   }
   baseName = convertNameToFile(diskName());
 
-  findMaximalDotGraph(m_startNode,QMIN(m_recDepth,m_maxDistance),baseName,
-                      thisDir,m_graphType,format,!isTBRank,m_graphType==DotNode::Inheritance);
 
-  if (format==BITMAP) // run dot to create a bitmap image
+  QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
+  QCString md5 = computeMd5Signature(m_startNode,        // root
+                                     m_graphType,        // gt
+                                     format,             // format
+                                     !isTBRank,          // lrRank
+                                     m_graphType==DotNode::Inheritance, // renderParent
+                                     QMIN(m_recDepth,m_maxDistance),    // maxDist
+                                     TRUE                               // backArrows
+                                    );
+  if (checkAndUpdateMd5Signature(baseName,md5) ||
+      !QFileInfo(baseName+".map").exists()
+     )
   {
-    QCString dotArgs(maxCmdLine);
-    QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
-    QCString imgName = baseName+"."+imgExt;
-    dotArgs.sprintf("-T%s \"%s.dot\" -o \"%s\"",
-                   imgExt.data(),baseName.data(),imgName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+    findMaximalDotGraph(m_startNode,                       // root
+                        QMIN(m_recDepth,m_maxDistance),    // maxDist
+                        baseName,                          // baseName
+                        thisDir,                           // thisDir
+                        m_graphType,                       // gt
+                        format,                            // format
+                        !isTBRank,                         // lrRank
+                        m_graphType==DotNode::Inheritance, // renderParents
+                        TRUE                               // backArrows
+                       );
+    if (format==BITMAP) // run dot to create a bitmap image
     {
-       err("Error: Problems running dot. Check your installation!\n");
-       QDir::setCurrent(oldDir);
-       return baseName;
-    }
-    checkDotResult(imgName);
-    if (generateImageMap)
-    {
-      // run dot again to create an image map
-      dotArgs.sprintf("-Timap \"%s.dot\" -o \"%s.map\"",baseName.data(),baseName.data());
+      QCString dotArgs(maxCmdLine);
+      QCString imgName = baseName+"."+imgExt;
+      dotArgs.sprintf("-T%s \"%s.dot\" -o \"%s\"",
+          imgExt.data(),baseName.data(),imgName.data());
       if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
       {
         err("Error: Problems running dot. Check your installation!\n");
         QDir::setCurrent(oldDir);
         return baseName;
       }
-      QCString mapLabel = convertNameToFile(m_startNode->m_label+"_"+mapName);
-      out << "<p><center><img src=\"" << baseName << "." 
-          << imgExt << "\" border=\"0\" usemap=\"#"
-          << mapLabel << "\" alt=\"";
-      switch (m_graphType)
+      checkDotResult(imgName);
+      if (generateImageMap)
       {
-        case DotNode::Collaboration:
-          out << "Collaboration graph";
-          break;
-        //case Interface:
-        //  out << "Interface dependency graph";
-        //  break;
-        case DotNode::Inheritance:
-          out << "Inheritance graph";
-          break;
-        default:
-          ASSERT(0);
-          break;
+        // run dot again to create an image map
+        dotArgs.sprintf("-Timap \"%s.dot\" -o \"%s.map\"",baseName.data(),baseName.data());
+        if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+        {
+          err("Error: Problems running dot. Check your installation!\n");
+          QDir::setCurrent(oldDir);
+          return baseName;
+        }
       }
-      out << "\"></center>" << endl;
-      QString tmpstr;
-      QTextOStream tmpout(&tmpstr);
-      convertMapFile(tmpout,baseName+".map");
-      if (!tmpstr.isEmpty())
-      {
-        out << "<map name=\"" << mapLabel << "\">" << endl;
-        out << tmpstr;
-        out << "</map>" << endl;
-      }
-      thisDir.remove(baseName+".map");
     }
+    else if (format==EPS) // run dot to create a .eps image
+    {
+      QCString dotArgs(maxCmdLine);
+      dotArgs.sprintf("-Tps \"%s.dot\" -o \"%s.eps\"",baseName.data(),baseName.data());
+      if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+      {
+        err("Error: Problems running dot. Check your installation!\n");
+        QDir::setCurrent(oldDir);
+        return baseName;
+      }
+      if (Config_getBool("USE_PDFLATEX"))
+      {
+        QCString epstopdfArgs(maxCmdLine);
+        epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
+            baseName.data(),baseName.data());
+        if (iSystem("epstopdf",epstopdfArgs,TRUE)!=0)
+        {
+          err("Error: Problems running epstopdf. Check your TeX installation!\n");
+          QDir::setCurrent(oldDir);
+          return baseName;
+        }
+      }
+
+    }
+    if (Config_getBool("DOT_CLEANUP")) thisDir.remove(baseName+".dot");
+  }
+
+  if (format==BITMAP && generateImageMap) // run dot to create a image map
+  {
+    QCString mapLabel = convertNameToFile(m_startNode->m_label+"_"+mapName);
+    out << "<p><center><img src=\"" << baseName << "." 
+      << imgExt << "\" border=\"0\" usemap=\"#"
+      << mapLabel << "\" alt=\"";
+    switch (m_graphType)
+    {
+      case DotNode::Collaboration:
+        out << "Collaboration graph";
+        break;
+      case DotNode::Inheritance:
+        out << "Inheritance graph";
+        break;
+      default:
+        ASSERT(0);
+        break;
+    }
+    out << "\"></center>" << endl;
+    QString tmpstr;
+    QTextOStream tmpout(&tmpstr);
+    convertMapFile(tmpout,baseName+".map");
+    if (!tmpstr.isEmpty())
+    {
+      out << "<map name=\"" << mapLabel << "\">" << endl;
+      out << tmpstr;
+      out << "</map>" << endl;
+    }
+    //thisDir.remove(baseName+".map");
   }
   else if (format==EPS) // run dot to create a .eps image
   {
-    QCString dotArgs(maxCmdLine);
-    dotArgs.sprintf("-Tps \"%s.dot\" -o \"%s.eps\"",baseName.data(),baseName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
-    {
-       err("Error: Problems running dot. Check your installation!\n");
-       QDir::setCurrent(oldDir);
-       return baseName;
-    }
-    int width,height;
-    if (!readBoundingBoxEPS(baseName+".eps",&width,&height))
-    {
-      err("Error: Could not extract bounding box from .eps!\n");
-      QDir::setCurrent(oldDir);
-      return baseName;
-    }
-    if (Config_getBool("USE_PDFLATEX"))
-    {
-      QCString epstopdfArgs(maxCmdLine);
-      epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
-                     baseName.data(),baseName.data());
-      if (iSystem("epstopdf",epstopdfArgs,TRUE)!=0)
+      int width,height;
+      if (!readBoundingBoxEPS(baseName+".eps",&width,&height))
       {
-         err("Error: Problems running epstopdf. Check your TeX installation!\n");
-         QDir::setCurrent(oldDir);
-         return baseName;
+        err("Error: Could not extract bounding box from .eps!\n");
+        QDir::setCurrent(oldDir);
+        return baseName;
       }
-    }
-    int maxWidth = 420; /* approx. page width in points */
-   
-    out << "\\begin{figure}[H]\n"
-           "\\begin{center}\n"
-           "\\leavevmode\n"
-           "\\includegraphics[width=" << QMIN(width/2,maxWidth) 
-                                      << "pt]{" << baseName << "}\n"
-           "\\end{center}\n"
-           "\\end{figure}\n";
+      int maxWidth = 420; /* approx. page width in points */
+      out << "\\begin{figure}[H]\n"
+        "\\begin{center}\n"
+        "\\leavevmode\n"
+        "\\includegraphics[width=" << QMIN(width/2,maxWidth) 
+        << "pt]{" << baseName << "}\n"
+        "\\end{center}\n"
+        "\\end{figure}\n";
   }
-  if (Config_getBool("DOT_CLEANUP")) thisDir.remove(baseName+".dot");
 
   QDir::setCurrent(oldDir);
   return baseName;
@@ -1762,69 +1910,109 @@ QCString DotInclDepGraph::writeGraph(QTextStream &out,
   baseName=convertNameToFile(baseName);
   QCString mapName=m_startNode->m_label.copy();
   if (m_inverse) mapName+="dep";
+  QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
 
-  findMaximalDotGraph(m_startNode,QMIN(m_recDepth,m_maxDistance),
-                      baseName,thisDir,DotNode::Dependency,format,
-                      FALSE,FALSE,!m_inverse);
-
-  if (format==BITMAP)
+  QCString md5 = computeMd5Signature(m_startNode,         // root
+                                     DotNode::Dependency, // gt
+                                     format,              // format
+                                     FALSE,               // lrRank
+                                     FALSE,               // renderParents
+                                     QMIN(m_recDepth,m_maxDistance), // maxDist
+                                     !m_inverse           // backArrows
+                                    );
+  if (checkAndUpdateMd5Signature(baseName,md5) ||
+      !QFileInfo(baseName+".map").exists()
+     )
   {
-    // run dot to create a bitmap image
-    QCString dotArgs(maxCmdLine);
-    QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
-    QCString imgName=baseName+"."+imgExt;
-    dotArgs.sprintf("-T%s \"%s.dot\" -o \"%s\"",
-                   imgExt.data(),baseName.data(),imgName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+    findMaximalDotGraph(m_startNode,                     // root
+                        QMIN(m_recDepth,m_maxDistance),  // maxDist
+                        baseName,                        // baseName
+                        thisDir,                         // thisDir
+                        DotNode::Dependency,             // gt
+                        format,                          // format
+                        FALSE,                           // lrRank
+                        FALSE,                           // renderParents
+                        !m_inverse                       // backArrows
+    );
+    if (format==BITMAP)
     {
-      err("Problems running dot. Check your installation!\n");
-      QDir::setCurrent(oldDir);
-      return baseName;
-    }
-    checkDotResult(imgName);
-
-    if (generateImageMap)
-    {
-      // run dot again to create an image map
-      dotArgs.sprintf("-Timap \"%s.dot\" -o \"%s.map\"",
-                      baseName.data(),baseName.data());
+      // run dot to create a bitmap image
+      QCString dotArgs(maxCmdLine);
+      QCString imgName=baseName+"."+imgExt;
+      dotArgs.sprintf("-T%s \"%s.dot\" -o \"%s\"",
+          imgExt.data(),baseName.data(),imgName.data());
       if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
       {
         err("Problems running dot. Check your installation!\n");
         QDir::setCurrent(oldDir);
         return baseName;
       }
+      checkDotResult(imgName);
 
-      out << "<p><center><img src=\"" << baseName << "." 
-          << imgExt << "\" border=\"0\" usemap=\"#"
-          << mapName << "_map\" alt=\"";
-      if (m_inverse) out << "Included by dependency graph"; else out << "Include dependency graph";
-      out << "\">";
-      out << "</center>" << endl;
-      QString tmpstr;
-      QTextOStream tmpout(&tmpstr);
-      convertMapFile(tmpout,baseName+".map");
-      if (!tmpstr.isEmpty())
+      if (generateImageMap)
       {
-        out << "<map name=\"" << mapName << "_map\">" << endl;
-        out << tmpstr;
-        out << "</map>" << endl;
+        // run dot again to create an image map
+        dotArgs.sprintf("-Timap \"%s.dot\" -o \"%s.map\"",
+            baseName.data(),baseName.data());
+        if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+        {
+          err("Problems running dot. Check your installation!\n");
+          QDir::setCurrent(oldDir);
+          return baseName;
+        }
+
+        //thisDir.remove(baseName+".map");
       }
-      thisDir.remove(baseName+".map");
+    }
+    else if (format==EPS)
+    {
+      // run dot to create a .eps image
+      QCString dotArgs(maxCmdLine);
+      dotArgs.sprintf("-Tps \"%s.dot\" -o \"%s.eps\"",
+          baseName.data(),baseName.data());
+      if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+      {
+        err("Problems running dot. Check your installation!\n");
+        QDir::setCurrent(oldDir);
+        return baseName;
+      }
+      if (Config_getBool("USE_PDFLATEX"))
+      {
+        QCString epstopdfArgs(maxCmdLine);
+        epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
+            baseName.data(),baseName.data());
+        if (iSystem("epstopdf",epstopdfArgs,TRUE)!=0)
+        {
+          err("Error: Problems running epstopdf. Check your TeX installation!\n");
+          QDir::setCurrent(oldDir);
+          return baseName;
+        }
+      }
+    }
+
+    if (Config_getBool("DOT_CLEANUP")) thisDir.remove(baseName+".dot");
+  }
+
+  if (format==BITMAP && generateImageMap)
+  {
+    out << "<p><center><img src=\"" << baseName << "." 
+      << imgExt << "\" border=\"0\" usemap=\"#"
+      << mapName << "_map\" alt=\"";
+    if (m_inverse) out << "Included by dependency graph"; else out << "Include dependency graph";
+    out << "\">";
+    out << "</center>" << endl;
+    QString tmpstr;
+    QTextOStream tmpout(&tmpstr);
+    convertMapFile(tmpout,baseName+".map");
+    if (!tmpstr.isEmpty())
+    {
+      out << "<map name=\"" << mapName << "_map\">" << endl;
+      out << tmpstr;
+      out << "</map>" << endl;
     }
   }
   else if (format==EPS)
   {
-    // run dot to create a .eps image
-    QCString dotArgs(maxCmdLine);
-    dotArgs.sprintf("-Tps \"%s.dot\" -o \"%s.eps\"",
-                   baseName.data(),baseName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
-    {
-      err("Problems running dot. Check your installation!\n");
-      QDir::setCurrent(oldDir);
-      return baseName;
-    }
     int width,height;
     if (!readBoundingBoxEPS(baseName+".eps",&width,&height))
     {
@@ -1832,32 +2020,17 @@ QCString DotInclDepGraph::writeGraph(QTextStream &out,
       QDir::setCurrent(oldDir);
       return baseName;
     }
-    if (Config_getBool("USE_PDFLATEX"))
-    {
-      QCString epstopdfArgs(maxCmdLine);
-      epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
-                     baseName.data(),baseName.data());
-      if (iSystem("epstopdf",epstopdfArgs,TRUE)!=0)
-      {
-         err("Error: Problems running epstopdf. Check your TeX installation!\n");
-         QDir::setCurrent(oldDir);
-         return baseName;
-      }
-    }
-    int maxWidth = 420; /* approx. page width in points */
-   
-    out << "\\begin{figure}[H]\n"
-           "\\begin{center}\n"
-           "\\leavevmode\n"
-           //"\\setlength{\\epsfxsize}{" << QMIN(width/2,maxWidth) << "pt}\n"
-           //"\\epsfbox{" << baseName << ".eps}\n"
-           "\\includegraphics[width=" << QMIN(width/2,maxWidth) 
-                                      << "pt]{" << baseName << "}\n"
-           "\\end{center}\n"
-           "\\end{figure}\n";
-  }
 
-  if (Config_getBool("DOT_CLEANUP")) thisDir.remove(baseName+".dot");
+    int maxWidth = 420; /* approx. page width in points */
+
+    out << "\\begin{figure}[H]\n"
+      "\\begin{center}\n"
+      "\\leavevmode\n"
+      "\\includegraphics[width=" << QMIN(width/2,maxWidth) 
+      << "pt]{" << baseName << "}\n"
+      "\\end{center}\n"
+      "\\end{figure}\n";
+  }
 
   QDir::setCurrent(oldDir);
   return baseName;
@@ -1886,25 +2059,10 @@ DotCallGraph::DotCallGraph(MemberDef *md,int maxRecursionDepth)
 {
   m_maxDistance = 0;
   m_recDepth = maxRecursionDepth;
-  if (md->getGroupDef())
-  {
-    m_diskName = md->getGroupDef()->getOutputFileBase()+"_"+md->getBodyAnchor();
-  }
-  else
-  {
-    m_diskName = md->getOutputFileBase()+"_"+md->anchor();
-  }
+  m_diskName = md->getOutputFileBase()+"_"+md->anchor();
   QCString uniqueId;
-  if (md->getGroupDef()) // member is in a group
-  {
-    uniqueId = md->getReference()+"$"+
-               md->getGroupDef()->getOutputFileBase()+"#"+md->getBodyAnchor();
-  }
-  else // ungrouped member
-  {
-    uniqueId = md->getReference()+"$"+
-               md->getOutputFileBase()+"#"+md->anchor();
-  }
+  uniqueId = md->getReference()+"$"+
+             md->getOutputFileBase()+"#"+md->anchor();
   m_startNode = new DotNode(m_curNodeNumber++,
                             md->qualifiedName(),
                             uniqueId.data(),
@@ -1937,70 +2095,106 @@ QCString DotCallGraph::writeGraph(QTextStream &out, GraphOutputFormat format,
   QDir thisDir;
 
   QCString baseName=m_diskName+"_cgraph";
-  //baseName=convertNameToFile(baseName);
   QCString mapName=baseName;
+  QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
 
-  findMaximalDotGraph(m_startNode,QMIN(m_recDepth,m_maxDistance),
-                      baseName,thisDir,DotNode::CallGraph,format,
-                      TRUE,FALSE,FALSE);
-
-  if (format==BITMAP)
+  QCString md5 = computeMd5Signature(m_startNode,         // root
+                                     DotNode::CallGraph,  // gt
+                                     format,              // format
+                                     TRUE,                // lrRank
+                                     FALSE,               // renderParents
+                                     QMIN(m_recDepth,m_maxDistance), // maxDist
+                                     FALSE                // backArrows
+                                    );
+  if (checkAndUpdateMd5Signature(baseName,md5) ||
+      !QFileInfo(baseName+".map").exists()
+     )
   {
-    // run dot to create a bitmap image
-    QCString dotArgs(maxCmdLine);
-    QCString imgExt = Config_getEnum("DOT_IMAGE_FORMAT");
-    QCString imgName=baseName+"."+imgExt;
-    dotArgs.sprintf("-T%s \"%s.dot\" -o \"%s\"",
-                   imgExt.data(),baseName.data(),imgName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+    findMaximalDotGraph(m_startNode,                      // root
+                        QMIN(m_recDepth,m_maxDistance),   // maxDist
+                        baseName,                         // baseName
+                        thisDir,                          // thisDir
+                        DotNode::CallGraph,               // gt
+                        format,                           // format
+                        TRUE,                             // lrRank
+                        FALSE,                            // renderParents
+                        FALSE                             // backArrows
+                       );
+    if (format==BITMAP)
     {
-      err("Problems running dot. Check your installation!\n");
-      QDir::setCurrent(oldDir);
-      return baseName;
-    }
-    checkDotResult(imgName);
-
-    if (generateImageMap)
-    {
-      // run dot again to create an image map
-      dotArgs.sprintf("-Timap \"%s.dot\" -o \"%s.map\"",
-                      baseName.data(),baseName.data());
+      // run dot to create a bitmap image
+      QCString dotArgs(maxCmdLine);
+      QCString imgName=baseName+"."+imgExt;
+      dotArgs.sprintf("-T%s \"%s.dot\" -o \"%s\"",
+          imgExt.data(),baseName.data(),imgName.data());
       if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
       {
         err("Problems running dot. Check your installation!\n");
         QDir::setCurrent(oldDir);
         return baseName;
       }
+      checkDotResult(imgName);
 
-      out << "<p><center><img src=\"" << baseName << "." 
-          << imgExt << "\" border=\"0\" usemap=\"#"
-          << mapName << "_map\" alt=\"";
-      out << "\">";
-      out << "</center>" << endl;
-      QString tmpstr;
-      QTextOStream tmpout(&tmpstr);
-      convertMapFile(tmpout,baseName+".map");
-      if (!tmpstr.isEmpty())
+      if (generateImageMap)
       {
-        out << "<map name=\"" << mapName << "_map\">" << endl;
-        out << tmpstr;
-        out << "</map>" << endl;
+        // run dot again to create an image map
+        dotArgs.sprintf("-Timap \"%s.dot\" -o \"%s.map\"",
+            baseName.data(),baseName.data());
+        if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+        {
+          err("Problems running dot. Check your installation!\n");
+          QDir::setCurrent(oldDir);
+          return baseName;
+        }
       }
-      thisDir.remove(baseName+".map");
     }
+    else if (format==EPS)
+    {
+      // run dot to create a .eps image
+      QCString dotArgs(maxCmdLine);
+      dotArgs.sprintf("-Tps \"%s.dot\" -o \"%s.eps\"",
+          baseName.data(),baseName.data());
+      if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
+      {
+        err("Problems running dot. Check your installation!\n");
+        QDir::setCurrent(oldDir);
+        return baseName;
+      }
+      if (Config_getBool("USE_PDFLATEX"))
+      {
+        QCString epstopdfArgs(maxCmdLine);
+        epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
+            baseName.data(),baseName.data());
+        if (iSystem("epstopdf",epstopdfArgs,TRUE)!=0)
+        {
+          err("Error: Problems running epstopdf. Check your TeX installation!\n");
+          QDir::setCurrent(oldDir);
+          return baseName;
+        }
+      }
+    }
+  }
+
+  if (format==BITMAP && generateImageMap)
+  {
+    out << "<p><center><img src=\"" << baseName << "." 
+        << imgExt << "\" border=\"0\" usemap=\"#"
+        << mapName << "_map\" alt=\"";
+    out << "\">";
+    out << "</center>" << endl;
+    QString tmpstr;
+    QTextOStream tmpout(&tmpstr);
+    convertMapFile(tmpout,baseName+".map");
+    if (!tmpstr.isEmpty())
+    {
+      out << "<map name=\"" << mapName << "_map\">" << endl;
+      out << tmpstr;
+      out << "</map>" << endl;
+    }
+    //thisDir.remove(baseName+".map");
   }
   else if (format==EPS)
   {
-    // run dot to create a .eps image
-    QCString dotArgs(maxCmdLine);
-    dotArgs.sprintf("-Tps \"%s.dot\" -o \"%s.eps\"",
-                   baseName.data(),baseName.data());
-    if (iSystem(Config_getString("DOT_PATH")+"dot",dotArgs)!=0)
-    {
-      err("Problems running dot. Check your installation!\n");
-      QDir::setCurrent(oldDir);
-      return baseName;
-    }
     int width,height;
     if (!readBoundingBoxEPS(baseName+".eps",&width,&height))
     {
@@ -2008,18 +2202,6 @@ QCString DotCallGraph::writeGraph(QTextStream &out, GraphOutputFormat format,
       QDir::setCurrent(oldDir);
       return baseName;
     }
-    if (Config_getBool("USE_PDFLATEX"))
-    {
-      QCString epstopdfArgs(maxCmdLine);
-      epstopdfArgs.sprintf("\"%s.eps\" --outfile=\"%s.pdf\"",
-                     baseName.data(),baseName.data());
-      if (iSystem("epstopdf",epstopdfArgs,TRUE)!=0)
-      {
-         err("Error: Problems running epstopdf. Check your TeX installation!\n");
-         QDir::setCurrent(oldDir);
-         return baseName;
-      }
-    }
     int maxWidth = 420; /* approx. page width in points */
    
     out << "\\begin{figure}[H]\n"
@@ -2049,16 +2231,8 @@ void DotCallGraph::buildGraph(DotNode *n,MemberDef *md,int distance)
       if (rmd->isFunction())
       {
         QCString uniqueId;
-        if (rmd->getGroupDef()) // member is in a group
-        {
-          uniqueId = rmd->getReference()+"$"+
-            rmd->getGroupDef()->getOutputFileBase()+"#"+rmd->getBodyAnchor();
-        }
-        else // ungrouped member
-        {
-          uniqueId=rmd->getReference()+"$"+
-            rmd->getOutputFileBase()+"#"+rmd->anchor();
-        }
+        uniqueId=rmd->getReference()+"$"+
+                 rmd->getOutputFileBase()+"#"+rmd->anchor();
         DotNode *bn  = m_usedNodes->find(uniqueId);
         if (bn) // file is already a node in the graph
         {
