@@ -2,7 +2,7 @@
  *
  * $Id: entry.cpp,v 1.29 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -74,6 +74,9 @@ Entry::Entry(const Entry &e)
   brief       = e.brief.copy();
   briefLine   = e.briefLine;
   briefFile   = e.briefFile.copy();
+  inbodyDocs  = e.inbodyDocs.copy();
+  inbodyLine  = e.inbodyLine;
+  inbodyFile  = e.inbodyFile.copy();
   inside      = e.inside.copy();
   fileName    = e.fileName.copy();
   startLine   = e.startLine;
@@ -84,6 +87,7 @@ Entry::Entry(const Entry &e)
   initializer = e.initializer;
   initLines   = e.initLines;
   callGraph   = e.callGraph;
+  objc        = e.objc;
   //todoId      = e.todoId;
   //testId      = e.testId;
   //bugId       = e.bugId;
@@ -233,6 +237,9 @@ void Entry::reset()
   brief.resize(0);
   briefFile.resize(0);
   briefLine=-1;
+  inbodyDocs.resize(0);
+  inbodyFile.resize(0);
+  inbodyLine=-1;
   inside.resize(0);
   fileName.resize(0);
   //scopeSpec.resize(0);
@@ -254,6 +261,7 @@ void Entry::reset()
   proto   = FALSE;
   explicitExternal = FALSE;
   memSpec  = 0;
+  objc = FALSE;
   subGrouping = TRUE;
   protection = Public;
   groupDocType = GROUPDOC_NORMAL;
@@ -285,6 +293,8 @@ int Entry::getSize()
   size+=relates.length()+1;
   size+=brief.length()+1;
   size+=briefFile.length()+1;
+  size+=inbodyDocs.length()+1;
+  size+=inbodyFile.length()+1;
   size+=inside.length()+1;
   size+=fileName.length()+1;
   size+=initializer.length()+1;
