@@ -3,7 +3,7 @@
  * $Id: $
  *
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -188,14 +188,14 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
   switch(s->type())
   {
     case DocVerbatim::Code: // fall though
-      m_t << "<div class=\"fragment\"><pre>"; 
+      m_t << "<pre class=\"fragment\"><div>"; 
       parseCode(m_ci,s->context(),s->text().latin1(),s->isExample(),s->exampleFile());
-      m_t << "</pre></div>"; 
+      m_t << "</div></pre>"; 
       break;
     case DocVerbatim::Verbatim: 
-      m_t << "<div class=\"fragment\"><pre>";
+      m_t << "<pre class=\"fragment\"><div>";
       filter(s->text());
-      m_t << "</pre></div>"; 
+      m_t << "</div></pre>"; 
       break;
     case DocVerbatim::HtmlOnly: 
       m_t << s->text(); 
@@ -293,10 +293,9 @@ void HtmlDocVisitor::visit(DocFormula *f)
 {
   if (m_hide) return;
   bool bDisplay = f->text().at(0)=='\\';
-  if (bDisplay) m_t << "<p class=formulaDsp>" << endl;
-  m_t << "<img class=formula"
-      << (bDisplay ? "Dsp" : "Inl");
-  m_t << " alt=\"";
+  if (bDisplay) m_t << "<p class=\"formulaDsp\">" << endl;
+  m_t << "<img class=\"formula" << (bDisplay ? "Dsp" : "Inl");
+  m_t << "\" alt=\"";
   filterQuotedCdataAttr(f->text());
   m_t << "\"";
   /// @todo cache image dimensions on formula generation and give height/width
@@ -660,7 +659,8 @@ void HtmlDocVisitor::visitPost(DocInternal *)
 void HtmlDocVisitor::visitPre(DocHRef *href)
 {
   if (m_hide) return;
-  m_t << "<a href=\"" << href->url() << "\">";
+  m_t << "<a href=\"" << href->url()  << "\""
+      << htmlAttribsToString(href->attribs()) << ">";
 }
 
 void HtmlDocVisitor::visitPost(DocHRef *) 
@@ -695,7 +695,7 @@ void HtmlDocVisitor::visitPre(DocImage *img)
     }
     m_t << "<div align=\"center\">" << endl;
     m_t << "<img src=\"" << img->name() << "\" alt=\"" 
-      << baseName << "\">" << endl;
+      << baseName << "\"" << htmlAttribsToString(img->attribs()) << ">" << endl;
     if (img->hasCaption())
     {
       m_t << "<p><strong>";
