@@ -2,7 +2,7 @@
  *
  * $Id: latexgen.cpp,v 1.58 2001/03/19 19:27:40 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -221,7 +221,7 @@ void LatexGenerator::init()
 
   t << endl
     << "clean:" << endl
-    << "\trm -f *.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out *.pdf" << endl;
+    << "\trm -f *.ps *.dvi *.aux *.toc *.idx *.ind *.ilg *.log *.out refman.pdf" << endl;
 }
 
 static void writeDefaultHeaderPart1(QTextStream &t)
@@ -247,7 +247,9 @@ static void writeDefaultHeaderPart1(QTextStream &t)
     "\\usepackage{multicol}\n"
     "\\usepackage{float}\n"
     "\\usepackage{textcomp}\n"
-    "\\usepackage{alltt}\n";
+    "\\usepackage{alltt}\n"
+    //"\\usepackage{ae,aecompl,aeguill}\n"
+    ;
   if (Config_getBool("PDF_HYPERLINKS")) 
   {
     t << "\\usepackage{times}" << endl;
@@ -427,12 +429,12 @@ void LatexGenerator::writeStyleSheetFile(QFile &f)
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2003";
+  //t << "Dimitri van Heesch \\copyright~1997-2004";
   writeDefaultStyleSheetPart2(t);
   t << theTranslator->trGeneratedAt( dateToString(TRUE), projectName );
   t << " doxygen";
   //t << " << theTranslator->trWrittenBy() << " ";
-  //t << "Dimitri van Heesch \\copyright~1997-2003";
+  //t << "Dimitri van Heesch \\copyright~1997-2004";
   writeDefaultStyleSheetPart3(t);
 }
 
@@ -800,14 +802,14 @@ void LatexGenerator::writeStyleInfo(int part)
       break;
     case 2:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2003";
+        //t << " Dimitri van Heesch \\copyright~1997-2004";
         t << "}]{}\n";
         writeDefaultStyleSheetPart2(t);
       }
       break;
     case 4:
       {
-        //t << " Dimitri van Heesch \\copyright~1997-2003";
+        //t << " Dimitri van Heesch \\copyright~1997-2004";
         writeDefaultStyleSheetPart3(t);
         endPlainFile();
       }
@@ -1233,7 +1235,7 @@ void LatexGenerator::codify(const char *str)
           MultiByte = FALSE;
           continue;
         }
-        if ( (uchar)c>=0x80 || (uchar)c<=0xff) // char in range [0x80..0xff]
+        if ((uchar)c>=0x80) // char in range [0x80..0xff]
         {
           t << (char)c;
           MultiByte = TRUE;
