@@ -2,7 +2,7 @@
  *
  * $Id: htmlgen.h,v 1.15 1998/11/28 11:33:19 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -237,13 +237,13 @@ class ManGenerator : public OutputGenerator
 
     void startMemberDocPrefixItem() {}
     void endMemberDocPrefixItem() {}
-    void startMemberDocName() {}
+    void startMemberDocName(bool) {}
     void endMemberDocName() {}
-    void startParameterType(bool) {}
+    void startParameterType(bool,const char *) {}
     void endParameterType() {}
     void startParameterName(bool) {}
     void endParameterName(bool,bool) {}
-    void startParameterList() {}
+    void startParameterList(bool) {}
     void endParameterList() {}
 
     void startFontClass(const char *) {}
