@@ -2,7 +2,7 @@
  *
  * $Id: memberlist.cpp,v 1.35 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -243,12 +243,12 @@ void MemberList::writePlainDeclarations(OutputList &ol,
                 if (md->isDetailedSectionLinkable())
                 {
                   ol.disableAllBut(OutputGenerator::Html);
-                  ol.endEmphasis();
+                  //ol.endEmphasis();
                   ol.docify(" ");
                   if (md->getGroupDef()!=0 && gd==0) // forward link to group
                   {
-                    ol.startTextLink(md->getGroupDef()->getOutputFileBase(),
-                        md->anchor());
+                    ol.startTextLink(md->getOutputFileBase(),
+                                     md->anchor());
                   }
                   else
                   {
@@ -256,7 +256,7 @@ void MemberList::writePlainDeclarations(OutputList &ol,
                   }
                   ol.parseText(theTranslator->trMore());
                   ol.endTextLink();
-                  ol.startEmphasis();
+                  //ol.startEmphasis();
                   ol.enableAll();
                 }
                 ol.endMemberDescription();
