@@ -2,7 +2,7 @@
  *
  * $Id: outputlist.cpp,v 1.35 2001/03/19 19:27:41 root Exp $
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -36,64 +36,17 @@ OutputList::OutputList(bool)
   outputs->setAutoDelete(TRUE);
 }
 
-//OutputList::OutputList(const OutputList *olist)
-//{
-//  //printf("OutputList::OutputList() deep copy\n");
-//  outputs = new QList<OutputGenerator>;
-//  outputs->setAutoDelete(TRUE);
-//
-//  QList<OutputGenerator> *ol=olist->outputs;
-//  OutputGenerator *og=ol->first();
-//  while (og)
-//  {
-//    OutputGenerator *ogc=og->copy();
-//    outputs->append(ogc); 
-//    if (og->isEnabled()) ogc->enable(); else ogc->disable();  
-//    og=ol->next();
-//  }
-//  //printf("OutputList::OutputList dst=%d res=%d\n",ol->count(),outputs->count());
-//}
-
 OutputList::~OutputList()
 {
   //printf("OutputList::~OutputList()\n");
   delete outputs;
 }
 
-//OutputList &OutputList::operator=(const OutputList &olist)
-//{
-//  if (this!=&olist)
-//  {
-//    QList<OutputGenerator> *ol=olist.outputs;
-//    OutputGenerator *ogsrc=ol->first();
-//    OutputGenerator *ogdst=outputs->first();
-//    //printf("OutputList::operator= src=%d dst=%d\n",outputs->count(),ol->count());
-//    while (ogdst)
-//    {
-//      ogdst=ogsrc->copy();
-//      ogsrc=ol->next();
-//      ogdst=outputs->next();
-//    }
-//  }
-//  return *this;
-//}
-
 void OutputList::add(const OutputGenerator *og)
 {
   if (og) outputs->append(og);
 }
 
-//HtmlGenerator *OutputList::getHtmlGenerator()
-//{
-//  OutputGenerator *og=outputs->first();
-//  while (og)
-//  {
-//    if (og->get(OutputGenerator::Html)) return (HtmlGenerator *)og;
-//    og=outputs->next();
-//  }
-//  return 0; // should not happen!
-//}
-
 void OutputList::disableAllBut(OutputGenerator::OutputType o)
 {
   OutputGenerator *og=outputs->first();
@@ -156,21 +109,6 @@ bool OutputList::isEnabled(OutputGenerator::OutputType o)
   return result;
 }
 
-//OutputList &OutputList::operator+=(const OutputList &outputList)
-//{
-//  OutputList *ol=(OutputList *)&outputList;
-//  OutputGenerator *ogsrc=ol->outputs->first();
-//  OutputGenerator *ogdst=outputs->first();
-//  //printf("OutputList::operator+= src=%d dst=%d\n",outputs->count(),ol->outputs->count());
-//  while (ogdst && ogsrc)
-//  {
-//    ogdst->append(ogsrc);
-//    ogsrc=ol->outputs->next();
-//    ogdst=outputs->next();
-//  }
-//  return *this;
-//}
-
 void OutputList::pushGeneratorState()
 {
   OutputGenerator *og=outputs->first();
@@ -335,6 +273,7 @@ FORALL4(const char *a1,const char *a2,const char *a3,bool a4,a1,a2,a3,a4)
 #endif
 FORALL2(int a1,bool a2,a1,a2)
 FORALL2(bool a1,HighlightedItem a2,a1,a2)
+FORALL2(bool a1,const char *a2,a1,a2)
 FORALL2(ParamListTypes a1,const char *a2,a1,a2)
 FORALL1(IndexSections a1,a1)
 FORALL2(const char *a1,const char *a2,a1,a2)
