@@ -3,7 +3,7 @@
  *
  *
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  * Authors: Dimitri van Heesch, Miguel Lobo.
  *
  * Permission to use, copy, modify, and distribute this software and its
@@ -1921,7 +1921,7 @@ void PerlModGenerator::generatePerlModForGroup(GroupDef *gd)
     GroupDef *sgd;
     for (gli.toFirst();(sgd=gli.current());++gli)
       m_output.openHash()
-	.addFieldQuotedString("title", gd->groupTitle())
+	.addFieldQuotedString("title", sgd->groupTitle())
 	.closeHash();
     m_output.closeList();
   }
