@@ -2,7 +2,7 @@
  *
  * 
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -536,6 +536,8 @@ class TranslatorFrench : public Translator
         case ClassDef::Struct:     result+="la structure "; break;
         case ClassDef::Union:      result+="l'union "; break;
         case ClassDef::Interface:  result+="l'interface "; break;
+        case ClassDef::Protocol:   result+="le protocol "; break; 
+        case ClassDef::Category:   result+="la catgorie "; break; 
         case ClassDef::Exception:  result+="l'exception "; break;
       }
 
@@ -694,6 +696,8 @@ class TranslatorFrench : public Translator
         case ClassDef::Struct:     result+="structure"; break;
         case ClassDef::Union:      result+="union"; break;
         case ClassDef::Interface:  result+="interface"; break;
+        case ClassDef::Protocol:   result+="protocol"; break; 
+        case ClassDef::Category:   result+="category"; break; 
         case ClassDef::Exception:  result+="exception"; break;
       }
       result+=" a t gnre  partir ";
