@@ -3,7 +3,7 @@
  * $Id: xml.cpp,v 1.2 2001/01/01 10:15:18 root Exp $
  *
  *
- * Copyright (C) 1997-2003 by Dimitri van Heesch.
+ * Copyright (C) 1997-2004 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -67,20 +67,28 @@ inline void writeXMLString(QTextStream &t,const char *s)
   t << convertToXML(s);
 }
 
-inline void writeXMLCodeString(QTextStream &t,const char *s)
+inline void writeXMLCodeString(QTextStream &t,const char *s, int &col)
 {
   char c;
   while ((c=*s++))
   {
     switch(c)
     {
-      case ' ':  t << "<sp/>";  break;
-      case '<':  t << "&lt;";   break;
-      case '>':  t << "&gt;";   break;
-      case '&':  t << "&amp;";  break;
-      case '\'': t << "&apos;"; break; 
-      case '"':  t << "&quot;"; break;
-      default:   t << c;        break;         
+      case '\t': 
+      { 
+	int spacesToNextTabStop = 
+          Config_getInt("TAB_SIZE") - (col%Config_getInt("TAB_SIZE")); 
+	col+=spacesToNextTabStop;
+	while (spacesToNextTabStop--) t << "<sp/>";
+	break;
+	}
+      case ' ':  t << "<sp/>"; col++;  break;
+      case '<':  t << "&lt;"; col++;   break;
+      case '>':  t << "&gt;"; col++;   break;
+      case '&':  t << "&amp;"; col++;  break;
+      case '\'': t << "&apos;"; col++; break; 
+      case '"':  t << "&quot;"; col++; break;
+      default:   t << c; col++;        break;         
     }
   } 
 }
@@ -88,24 +96,6 @@ inline void writeXMLCodeString(QTextStream &t,const char *s)
 
 static void writeXMLHeader(QTextStream &t)
 {
-  //QCString dtdName = Config_getString("XML_DTD");
-  //QCString schemaName = Config_getString("XML_SCHEMA");
-  //t << "<?xml version='1.0' encoding='" << theTranslator->idLanguageCharset()
-  //  << "' standalone='";
-  //if (dtdName.isEmpty() && schemaName.isEmpty()) t << "yes"; else t << "no";
-  //t << "'?>" << endl;
-  //if (!dtdName.isEmpty())
-  //{
-  //  t << "<!DOCTYPE doxygen SYSTEM \"doxygen.dtd\">" << endl;
-  //}
-  //t << "<doxygen ";
-  //if (!schemaName.isEmpty())
-  //{
-  //  t << "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ";
-  //  t << "xsi:noNamespaceSchemaLocation=\"doxygen.xsd\" ";
-  //}
-  //t << "version=\"" << versionString << "\">" << endl;
-
   t << "<?xml version='1.0' encoding='" << theTranslator->idLanguageCharset()
     << "' standalone='no'?>" << endl;;
   t << "<doxygen xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" ";
@@ -113,6 +103,38 @@ static void writeXMLHeader(QTextStream &t)
   t << "version=\"" << versionString << "\">" << endl;
 }
 
+static void writeCombineScript()
+{
+  QCString outputDirectory = Config_getString("XML_OUTPUT");
+  QCString fileName=outputDirectory+"/combine.xslt";
+  QFile f(fileName);
+  if (!f.open(IO_WriteOnly))
+  {
+    err("Cannot open file %s for writing!\n",fileName.data());
+    return;
+  }
+  QTextStream t(&f);
+  t.setEncoding(QTextStream::Latin1);
+
+  t <<
+  "<!-- XSLT script to combine the generated output into a single file. \n"
+  "     If you have xsltproc you could use:\n"
+  "     xsltproc combine.xslt index.xml >all.xml\n"
+  "-->\n"
+  "<xsl:stylesheet xmlns:xsl=\"http://www.w3.org/1999/XSL/Transform\" version=\"1.0\">\n"
+  "  <xsl:output method=\"xml\" version=\"1.0\" indent=\"yes\" standalone=\"yes\" />\n"
+  "  <xsl:template match=\"/\">\n"
+  "    <doxygen version=\"{doxygenindex/@version}\">\n"
+  "      <!-- Load all doxgen generated xml files -->\n"
+  "      <xsl:for-each select=\"doxygenindex/compound\">\n"
+  "        <xsl:copy-of select=\"document( concat( @refid, '.xml' ) )/doxygen/*\" />\n"
+  "      </xsl:for-each>\n"
+  "    </doxygen>\n"
+  "  </xsl:template>\n"
+  "</xsl:stylesheet>\n";
+
+}
+
 void writeXMLLink(QTextStream &t,const char *extRef,const char *compoundId,
                   const char *anchorId,const char *text)
 {
@@ -217,7 +239,7 @@ class XMLCodeGenerator : public BaseCodeDocInterface
         m_t << "<highlight class=\"normal\">";
         m_normalHLNeedStartTag=FALSE;
       }
-      writeXMLCodeString(m_t,text);
+      writeXMLCodeString(m_t,text,col);
     }
     void writeCodeLink(const char *ref,const char *file,
                                const char *anchor,const char *name) 
@@ -229,6 +251,7 @@ class XMLCodeGenerator : public BaseCodeDocInterface
         m_normalHLNeedStartTag=FALSE;
       }
       writeXMLLink(m_t,ref,file,anchor,name);
+      col+=strlen(name);
     }
     void startCodeLine() 
     {
@@ -256,6 +279,7 @@ class XMLCodeGenerator : public BaseCodeDocInterface
       }
       m_t << ">"; 
       m_insideCodeLine=TRUE;
+      col=0;
     }
     void endCodeLine() 
     {
@@ -333,6 +357,7 @@ class XMLCodeGenerator : public BaseCodeDocInterface
     QCString m_external;
     int m_lineNumber;
     bool m_isMemberRef;
+    int col;
 
     bool m_insideCodeLine;
     bool m_normalHLNeedStartTag;
@@ -736,6 +761,9 @@ static void generateXMLForMember(MemberDef *md,QTextStream &ti,QTextStream &t,De
   t << "        <detaileddescription>" << endl;
   writeXMLDocBlock(t,md->docFile(),md->docLine(),md->getOuterScope(),md,md->documentation());
   t << "        </detaileddescription>" << endl;
+  t << "        <inbodydescription>" << endl;
+  writeXMLDocBlock(t,md->docFile(),md->inbodyLine(),md->getOuterScope(),md,md->inbodyDocumentation());
+  t << "        </inbodydescription>" << endl;
   if (md->getDefLine()!=-1)
   {
     t << "        <location file=\"" 
@@ -803,12 +831,9 @@ static void writeListOfAllMembers(ClassDef *cd,QTextStream &t)
     for (mii.toFirst();(mi=mii.current());++mii)
     {
       MemberDef *md=mi->memberDef;
-      ClassDef  *cd=md->getClassDef();
-      Definition *d=md->getGroupDef();
-      if (d==0) d = cd;
       Protection prot = mi->prot;
       Specifier virt=md->virtualness();
-      t << "      <member refid=\"" << d->getOutputFileBase() << "_1" <<
+      t << "      <member refid=\"" << md->getOutputFileBase() << "_1" <<
         md->anchor() << "\" prot=\"";
       switch (prot)
       {
@@ -876,7 +901,15 @@ static void generateXMLForClass(ClassDef *cd,QTextStream &ti)
   writeXMLHeader(t);
   t << "  <compounddef id=\"" 
     << cd->getOutputFileBase() << "\" kind=\"" 
-    << cd->compoundTypeString() << "\">" << endl;
+    << cd->compoundTypeString() << "\" prot=\"";
+  switch (cd->protection())
+  {
+    case Public:    t << "public";    break;
+    case Protected: t << "protected"; break;
+    case Private:   t << "private";   break;
+    case Package:   t << "package";   break;
+  }
+  t << "\">" << endl;
   t << "    <compoundname>"; 
   writeXMLString(t,cd->name()); 
   t << "</compoundname>" << endl;
@@ -990,6 +1023,11 @@ static void generateXMLForClass(ClassDef *cd,QTextStream &ti)
   generateXMLSection(cd,ti,t,&cd->proSlots,"protected-slot");
   generateXMLSection(cd,ti,t,&cd->proStaticMethods,"protected-static-func");
   generateXMLSection(cd,ti,t,&cd->proStaticAttribs,"protected-static-attrib");
+  generateXMLSection(cd,ti,t,&cd->pacTypes,"package-type");
+  generateXMLSection(cd,ti,t,&cd->pacMethods,"package-func");
+  generateXMLSection(cd,ti,t,&cd->pacAttribs,"package-attrib");
+  generateXMLSection(cd,ti,t,&cd->pacStaticMethods,"package-static-func");
+  generateXMLSection(cd,ti,t,&cd->pacStaticAttribs,"package-static-attrib");
   generateXMLSection(cd,ti,t,&cd->priTypes,"private-type");
   generateXMLSection(cd,ti,t,&cd->priMethods,"private-func");
   generateXMLSection(cd,ti,t,&cd->priAttribs,"private-attrib");
@@ -1249,9 +1287,12 @@ static void generateXMLForFile(FileDef *fd,QTextStream &ti)
   t << "    <detaileddescription>" << endl;
   writeXMLDocBlock(t,fd->docFile(),fd->docLine(),fd,0,fd->documentation());
   t << "    </detaileddescription>" << endl;
-  t << "    <programlisting>" << endl;
-  writeXMLCodeBlock(t,fd);
-  t << "    </programlisting>" << endl;
+  if (Config_getBool("XML_PROGRAMLISTING"))
+  {
+    t << "    <programlisting>" << endl;
+    writeXMLCodeBlock(t,fd);
+    t << "    </programlisting>" << endl;
+  }
   t << "    <location file=\"" << fd->getDefFileName() << "\"/>" << endl;
   t << "  </compounddef>" << endl;
   t << "</doxygen>" << endl;
@@ -1335,8 +1376,12 @@ static void generateXMLForGroup(GroupDef *gd,QTextStream &ti)
     PageDef *pd;
     for (pli.toFirst();(pd=pli.current());++pli)
     {
-      t << "    <innerpage refid=\"" << pd->getOutputFileBase()
-        << "\">" << convertToXML(pd->title()) << "</innerpage>" << endl;
+      t << "    <innerpage refid=\"" << pd->getOutputFileBase();
+      if (pd->getGroupDef())
+      {
+        t << "_" << pd->name();
+      }
+      t << "\">" << convertToXML(pd->title()) << "</innerpage>" << endl;
     }
   }
 
@@ -1347,7 +1392,7 @@ static void generateXMLForGroup(GroupDef *gd,QTextStream &ti)
     GroupDef *sgd;
     for (gli.toFirst();(sgd=gli.current());++gli)
     {
-      t << "    <innergroup refid=\"" << gd->getOutputFileBase()
+      t << "    <innergroup refid=\"" << sgd->getOutputFileBase()
         << "\">" << convertToXML(sgd->groupTitle()) 
         << "</innergroup>" << endl;
     }
@@ -1379,19 +1424,25 @@ static void generateXMLForGroup(GroupDef *gd,QTextStream &ti)
   ti << "  </compound>" << endl;
 }
 
-static void generateXMLForPage(PageDef *pd,QTextStream &ti)
+static void generateXMLForPage(PageDef *pd,QTextStream &ti,bool isExample)
 {
   // + name
   // + title
   // + documentation
 
+  const char *kindName = isExample ? "example" : "page";
+
   if (pd->isReference()) return;
   
   QCString pageName = pd->getOutputFileBase();
+  if (pd->getGroupDef())
+  {
+    pageName+=(QCString)"_"+pd->name();
+  }
   if (pageName=="index") pageName="indexpage"; // to prevent overwriting the generated index page.
   
   ti << "  <compound refid=\"" << pageName
-     << "\" kind=\"page\"><name>" << convertToXML(pd->name()) 
+     << "\" kind=\"" << kindName << "\"><name>" << convertToXML(pd->name()) 
      << "</name>" << endl;
   
   QCString outputDirectory = Config_getString("XML_OUTPUT");
@@ -1407,7 +1458,7 @@ static void generateXMLForPage(PageDef *pd,QTextStream &ti)
   t.setEncoding(QTextStream::Latin1);
   writeXMLHeader(t);
   t << "  <compounddef id=\"" << pageName;
-  t << "\" kind=\"page\">" << endl;
+  t << "\" kind=\"" << kindName << "\">" << endl;
   t << "    <compoundname>" << convertToXML(pd->name()) 
     << "</compoundname>" << endl;
   SectionInfo *si = Doxygen::sectionDict.find(pd->name());
@@ -1416,8 +1467,18 @@ static void generateXMLForPage(PageDef *pd,QTextStream &ti)
     t << "    <title>" << convertToXML(si->title) << "</title>" << endl;
   }
   t << "    <detaileddescription>" << endl;
-  writeXMLDocBlock(t,pd->docFile(),pd->docLine(),pd,0,pd->documentation());
+  if (isExample)
+  {
+    writeXMLDocBlock(t,pd->docFile(),pd->docLine(),pd,0,
+        pd->documentation()+"\n\\include "+pd->name());
+  }
+  else
+  {
+    writeXMLDocBlock(t,pd->docFile(),pd->docLine(),pd,0,
+        pd->documentation());
+  }
   t << "    </detaileddescription>" << endl;
+
   t << "  </compounddef>" << endl;
   t << "</doxygen>" << endl;
 
@@ -1543,21 +1604,34 @@ void generateXML()
     msg("Generating XML output for group %s\n",gd->name().data());
     generateXMLForGroup(gd,t);
   }
-  PageSDict::Iterator pdi(*Doxygen::pageSDict);
-  PageDef *pd=0;
-  for (pdi.toFirst();(pd=pdi.current());++pdi)
   {
-    msg("Generating XML output for page %s\n",pd->name().data());
-    generateXMLForPage(pd,t);
+    PageSDict::Iterator pdi(*Doxygen::pageSDict);
+    PageDef *pd=0;
+    for (pdi.toFirst();(pd=pdi.current());++pdi)
+    {
+      msg("Generating XML output for page %s\n",pd->name().data());
+      generateXMLForPage(pd,t,FALSE);
+    }
+  }
+  {
+    PageSDict::Iterator pdi(*Doxygen::exampleSDict);
+    PageDef *pd=0;
+    for (pdi.toFirst();(pd=pdi.current());++pdi)
+    {
+      msg("Generating XML output for example %s\n",pd->name().data());
+      generateXMLForPage(pd,t,TRUE);
+    }
   }
   if (Doxygen::mainPage)
   {
     msg("Generating XML output for the main page\n");
-    generateXMLForPage(Doxygen::mainPage,t);
+    generateXMLForPage(Doxygen::mainPage,t,FALSE);
   }
 
   //t << "  </compoundlist>" << endl;
   t << "</doxygenindex>" << endl;
+
+  writeCombineScript();
 }
 
 
